--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    02:19:35 11/09/05
-- Design Name:    
-- Module Name:    stream - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity stream is
  port (
    CLK : in std_logic;
    funct : in  std_logic_vector(3 downto 0);
    put_left_word, put_right_word : in  std_logic_vector(31 downto 0);
    shift : in  std_logic_vector(4 downto 0);
    get_word : out  std_logic_vector (31 downto 0);
    complete : out std_logic;
    USBD : inout std_logic_vector(7 downto 0);
    USBRDX : out std_logic;
    USBWR : out std_logic;
    USBTXEX : in std_logic;
    USBRXFX : in std_logic);
end stream;

architecture Behavioral of stream is
  signal count : std_logic_vector(2 downto 0) := "000";
  signal complete_buf : std_logic;
begin

  get_word <=
    put_left_word + put_right_word when funct = "0000" else
    put_left_word - put_right_word when funct = "0001" else
    shl(put_left_word, shift) when funct = "0010" else
    shr(put_left_word, shift) when funct = "0011" else
    put_left_word and put_right_word when funct = "0100" else
    put_left_word or put_right_word when funct = "0101" else
    put_left_word xor put_right_word when funct = "0110" else
    put_left_word xnor put_right_word when funct = "0111" else
    "000000000000000000000000" & USBD when funct = "1110" else
    put_left_word when funct = "1111" else
    "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";

  complete_buf <= not funct(3) or (count(2) and count(1) and count(0));

  complete <= complete_buf;

  USBD <=
    "ZZZZZZZZ" when funct = "1110" else
    put_left_word(7 downto 0) when funct = "1111" else
    "XXXXXXXX";

   
 process (CLK)
  begin  -- process
    if CLK'event and CLK = '1' then  -- rising clock edge
      if complete_buf = '1' then
        count <= "000";
      elsif (count /= "100" or funct /= "1110" or USBRXFX = '0') and (count /= "011" or funct /= "1111" or USBTXEX = '0') then
        count <= count + 1;
      end if;      
    end if;
  end process;

  process (CLK)
  begin  -- process O
    if CLK'event and CLK = '1' then  -- rising clock edge
      case funct is
        when "0000" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0001" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0010" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0011" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0100" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0101" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0110" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "0111" =>
          USBRDX <= '1';
          USBWR <= '0';
        when "1110" =>
          case count is
            when "000" => USBRDX <= '1';
            when "001" => USBRDX <= '1';
            when "010" => USBRDX <= '1';
            when "011" => USBRDX <= '1';
            when "100" => USBRDX <= '1';
            when "101" => USBRDX <= '0';
            when "110" => USBRDX <= '0';
            when "111" => USBRDX <= '0';
            when others => USBRDX <= 'X';        
          end case;
          USBWR <= '0';
        when "1111" =>
          USBRDX <= '1';
          case count is
            when "000" => USBWR <= '0';
            when "001" => USBWR <= '0';
            when "010" => USBWR <= '0';
            when "011" => USBWR <= '0';
            when "100" => USBWR <= '1';
            when "101" => USBWR <= '1';
            when "110" => USBWR <= '1';
            when "111" => USBWR <= '0';
            when others => USBWR <= 'X';        
          end case;
        when others =>
          USBRDX <= 'X';
          USBWR <= 'X';
      end case;
    end if;
  end process;
  
end Behavioral;
